/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007,2008  Arto Karppinen (arto.karppinen@iki.fi),
 * Joni Kokko (t5kojo01@students.oamk.fi),
 * Sami Mäki (t5masa02@students.oamk.fi),
 * Harri Vattulainen (t5vaha01@students.oamk.fi)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_webdb_cache.h"
#include "irreco_webdb_client.h"



/*
 * PURPOCE OF FILE. TODO TODO TODO
 * Caches XML-RPC calls, enables error handling and retries failed calls.
 */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * Create new IrrecoWebdb Object
 */
IrrecoWebdbCache *irreco_webdb_cache_new()
{
	IrrecoWebdbCache *self;
	IRRECO_ENTER
	
	self = g_slice_new0(IrrecoWebdbCache);
	self->private = irreco_webdb_client_new();
	self->error_msg = g_string_new("");
	self->loop = irreco_retry_loop_new(IRRECO_SECONDS_TO_USEC(0.3),
					   IRRECO_SECONDS_TO_USEC(3));
	self->conf_hash = g_hash_table_new_full(g_int_hash, g_int_equal, NULL,
		(GDestroyNotify) irreco_webdb_conf_free);
	IRRECO_RETURN_PTR(self);
}

/**
 * Free resources of IrrecoWebdbCache Object.
 */
void irreco_webdb_cache_free(IrrecoWebdbCache *self)
{
	IRRECO_ENTER
	g_string_free(self->error_msg, TRUE);
	self->error_msg = NULL;
	irreco_retry_loop_free(self->loop);
	self->loop = NULL;
	irreco_webdb_client_free(self->private);
	self->private = NULL;
	g_slice_free(IrrecoWebdbCache, self);
	g_hash_table_destroy(self->conf_hash);
	self->conf_hash = NULL;
	IRRECO_RETURN
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private Functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

/**
 * A simple test on Irreco WebDB to test if WebDB returns correct and 
 * meaningfull results.
 *
 * This function generates two numbers and asks WebDB to sum them. If the sum
 * if correct, then our connection to the WebDB seems to be working.
 *
 * @return TRUE on success, FALSE otherwise.
 */
static gboolean irreco_webdb_cache_test(IrrecoWebdbCache *self)
{
	IrrecoWebdbClient *client;
	GRand *rand;
	gint32 num_a;
	gint32 num_b;
	gint sum;
	IRRECO_ENTER
	
	if (self->test_ok) IRRECO_RETURN_BOOL(self->test_ok);
	
	rand = g_rand_new();
	client = (IrrecoWebdbClient *) self->private;
	
	IRRECO_RETRY_LOOP_START(self->loop)
		num_a = g_rand_int_range(rand, 1, 1000);
		num_b = g_rand_int_range(rand, 1, 1000);
		self->test_ok = irreco_webdb_client_sum(
			client, num_a, num_b, &sum);
		
		if (self->test_ok) {
			if (sum == num_a + num_b) {
				break;
			} else {
				g_string_printf(self->error_msg,
						"Got invalid result from "
						"sum method. %i + %i = %i. "
						"Expected %i.",
						num_a, num_b, sum, 
						num_a + num_b);
				IRRECO_PRINTF("%s\n", self->error_msg->str);
				self->test_ok = FALSE;
			}
		} else {
			irreco_webdb_client_get_error_msg(
				client, self->error_msg);
		}
	IRRECO_RETRY_LOOP_END(self->loop)
	
	g_rand_free(rand);
	IRRECO_RETURN_BOOL(self->test_ok);
}

/**
 * Shows error message in case categories list is empty.
 *
 * @return TRUE if categories is not empty, FALSE otherwise.
 */
static gboolean irreco_webdb_cache_verify_category(IrrecoWebdbCache *self)
{
	IRRECO_ENTER
	if (self->categories == NULL) {
		g_string_printf(self->error_msg, "list of categories is NULL");
		IRRECO_PRINTF("%s\n", self->error_msg->str);
		IRRECO_RETURN_BOOL(FALSE)		
	}
	IRRECO_RETURN_BOOL(TRUE)
}

/**
 * Shows error message in case manufacturers list is empty.
 *
 * @return TRUE if manufacturers list contains data, FALSE otherwise.
 */
static gboolean 
irreco_webdb_cache_verify_manufacturer(IrrecoWebdbCache *self,
				       const gchar *category,
				       IrrecoStringTable **manufactures)
{
	IRRECO_ENTER
	if (!irreco_string_table_get(self->categories, category,
				     (gpointer*) manufactures)) {
		g_string_printf(self->error_msg,
				"list of manufacturers is NULL");
		IRRECO_PRINTF("%s\n", self->error_msg->str);
		IRRECO_RETURN_BOOL(FALSE)		
	}
	IRRECO_RETURN_BOOL(TRUE)
}


/**
 * Shows error message in case models list is empty.
 *
 * @return TRUE if models list contains data, FALSE otherwise.
 */
static gboolean irreco_webdb_cache_verify_model(IrrecoWebdbCache *self,
						IrrecoStringTable *manuf_list,
						const gchar *manufacturer,
						IrrecoStringTable **models)
{
	IRRECO_ENTER
	if (!irreco_string_table_get(manuf_list, manufacturer,
				     (gpointer*) models)) {
		g_string_printf(self->error_msg, "list of models is NULL");
		IRRECO_PRINTF("%s\n", self->error_msg->str);
		IRRECO_RETURN_BOOL(FALSE)		
	}
	IRRECO_RETURN_BOOL(TRUE)
}

/**
 * Shows error message in case configurations list is empty.
 *
 * @return TRUE if configurations list contains data, FALSE otherwise.
 */
static gboolean irreco_webdb_cache_verify_config(IrrecoWebdbCache *self,
						 IrrecoStringTable *models,
						 const gchar *model,
						 IrrecoStringTable **configs)
{
	IRRECO_ENTER
	if (!irreco_string_table_get(models, model, (gpointer*) configs)) {
		g_string_printf(self->error_msg,
				"list of configurations is NULL");
		IRRECO_PRINTF("%s\n", self->error_msg->str);
		IRRECO_RETURN_BOOL(FALSE)		
	}
	IRRECO_RETURN_BOOL(TRUE)
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Functions                                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/**
 * Adds new user (name, email, password) to database.
 *
 * @return TRUE if user is added succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_add_user(IrrecoWebdbCache *self, 
				     const gchar *name,
				     const gchar *email,
				     const gchar *passwd)
{
	IrrecoWebdbClient *client;
	
	IRRECO_ENTER
	
	client = (IrrecoWebdbClient *) self->private;
	
	IRRECO_RETRY_LOOP_START(self->loop)
		/* test connection to webdb */
		if (irreco_webdb_cache_test(self) == FALSE){
			g_string_printf(self->error_msg,
						"Failed cache self test.");
			IRRECO_PRINTF("%s\n", self->error_msg->str);
			IRRECO_RETURN_BOOL(FALSE);
		}
	
		if(irreco_webdb_client_add_user(client, name, email, passwd)) {
			IRRECO_RETURN_BOOL(TRUE);
		} else {
			/* irreco_webdb_client_add_user failed, get err msg */
			irreco_webdb_client_get_error_msg(client, 
							  self->error_msg);
			IRRECO_RETURN_BOOL(FALSE);
		}
	IRRECO_RETRY_LOOP_END(self->loop)

	IRRECO_RETURN_BOOL(FALSE);
}

/**
 * Adds new configuration to database.
 *
 * @return TRUE if configuration is uploaded succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_upload_configuration(IrrecoWebdbCache *self,
						 const gchar *backend,
						 const gchar *category,
						 const gchar *file_hash,
						 const gchar *file_name,
						 const gchar *manufacturer,
						 const gchar *model,
						 const gchar *password,
						 const gchar *user,
						 const gchar *data)
{
	IrrecoWebdbClient *client;
	
	IRRECO_ENTER
	
	client = (IrrecoWebdbClient *) self->private;
	
	IRRECO_RETRY_LOOP_START(self->loop)
		/* test connection to webdb */
		if (irreco_webdb_cache_test(self) == FALSE){
			g_string_printf(self->error_msg,
						"Failed cache self test.");
			IRRECO_PRINTF("%s\n", self->error_msg->str);
			IRRECO_RETURN_BOOL(FALSE);
		}
	
		if(irreco_webdb_client_upload_configuration(client, backend, 
					category, file_hash, file_name, 
					manufacturer, model, password, user,
					data)) {

			IRRECO_RETURN_BOOL(TRUE);
		} else {
			/* irreco_webdb_client_upload_configuration failed, 
			   get err msg */
			irreco_webdb_client_get_error_msg(client, 
							  self->error_msg);
			IRRECO_RETURN_BOOL(FALSE);
		}
	IRRECO_RETRY_LOOP_END(self->loop)

	IRRECO_RETURN_BOOL(FALSE);
}

/**
 * Extracts error message from IrrecoWebdbCache object.
 *
 * @return error message as string.
 */
const gchar *irreco_webdb_cache_get_error(IrrecoWebdbCache *self)
{
	IRRECO_ENTER
	IRRECO_RETURN_STR(self->error_msg->str);
}

/**
 * Fetches categories from DB
 *
 * @return TRUE if categories are fetched succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_get_categories(IrrecoWebdbCache *self, 
					   IrrecoStringTable **categories)
{
	IRRECO_ENTER
	
	if (self->categories == NULL) {
		gboolean success = FALSE;
		IrrecoWebdbClient *client = (IrrecoWebdbClient *) self->private;
		
		IRRECO_RETRY_LOOP_START(self->loop)
			if (irreco_webdb_cache_test(self) == FALSE) break;
			success = irreco_webdb_client_get_categories(
				client, &self->categories);
			if (success) {
			    break;
			}
			irreco_webdb_client_get_error_msg(
				client, self->error_msg);
		IRRECO_RETRY_LOOP_END(self->loop)
		
		if (success == FALSE) IRRECO_RETURN_BOOL(FALSE);
		
		irreco_string_table_sort_abc (self->categories);
	}
	
	*categories = self->categories;
	IRRECO_RETURN_BOOL(TRUE)
}

/**
 * Fetches all categories from DB.
 *
 * @param all_categories must be freed if return value is TRUE.
 * @return TRUE if categories are fetched succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_get_all_categories(IrrecoWebdbCache *self, 
					   IrrecoStringTable **all_categories)
{
	gboolean success = FALSE;
	IrrecoWebdbClient *client = (IrrecoWebdbClient *) self->private;
	IRRECO_ENTER
	
	
	IRRECO_RETRY_LOOP_START(self->loop)
		if (irreco_webdb_cache_test(self) == FALSE) break;
		success = irreco_webdb_client_get_all_categories(
			client, all_categories);
		if (success) {
		    break;
		}
		irreco_webdb_client_get_error_msg(
			client, self->error_msg);
	IRRECO_RETRY_LOOP_END(self->loop)
	
	if (success == FALSE) IRRECO_RETURN_BOOL(FALSE);
	
	irreco_string_table_sort_abc(*all_categories);
	
	IRRECO_RETURN_BOOL(TRUE)
}

/**
 * Fetches manufacturers from DB.
 *
 * @param category contains category in which type manufacturers are returned.
 * @return TRUE if manufacturers are fetched succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_get_manufacturers(IrrecoWebdbCache *self,
					      const gchar *category, 
					      IrrecoStringTable **manufacturers)
{
	IrrecoStringTable *manufacturer_list;
	IRRECO_ENTER
	
	if (!irreco_webdb_cache_verify_category(self) ||
	    !irreco_webdb_cache_verify_manufacturer(self, category,
						    &manufacturer_list)) {
		IRRECO_RETURN_BOOL(FALSE)		
	}

	if(manufacturer_list == NULL) {
		gboolean success = FALSE;
		IrrecoWebdbClient *client = (IrrecoWebdbClient *) self->private;
		
		IRRECO_RETRY_LOOP_START(self->loop)
			if (irreco_webdb_cache_test(self) == FALSE) break;
			success = irreco_webdb_client_get_manufacturers(
			client, category, &manufacturer_list);
			
			if (success) break;
			irreco_webdb_client_get_error_msg(
				client, self->error_msg);
		IRRECO_RETRY_LOOP_END(self->loop)
		
		if (success == FALSE) IRRECO_RETURN_BOOL(FALSE);
		
		irreco_string_table_sort_abc (manufacturer_list);
		irreco_string_table_change_data(self->categories, category,
						manufacturer_list);
	}
	
	irreco_string_table_get(self->categories, category,
				(gpointer *) manufacturers);
	IRRECO_RETURN_BOOL(TRUE)
}

/**
 * Fetches all manufacturers from DB.
 *
 * @param all_manufacturers must be released if function returns TRUE.
 * @return TRUE if manufacturers are fetched succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_get_all_manufacturers(IrrecoWebdbCache *self,
					      IrrecoStringTable **all_manufacturers)
{
	gboolean success = FALSE;
	IrrecoWebdbClient *client = (IrrecoWebdbClient *) self->private;
	IRRECO_ENTER

	
	IRRECO_RETRY_LOOP_START(self->loop)
		if (irreco_webdb_cache_test(self) == FALSE) break;
		success = irreco_webdb_client_get_all_manufacturers(
			client, all_manufacturers);
		if (success) {
		    break;
		}
		irreco_webdb_client_get_error_msg(
			client, self->error_msg);
	IRRECO_RETRY_LOOP_END(self->loop)
	
	if (success == FALSE) IRRECO_RETURN_BOOL(FALSE);
	
	irreco_string_table_sort_abc(*all_manufacturers);
	
	IRRECO_RETURN_BOOL(TRUE)
}

/**
 * Fetches models belonging given category and manufacturer.
 *
 * @param models must be freed if function returns TRUE.
 * @return TRUE if models are fetched succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_get_models(IrrecoWebdbCache *self,
				       const gchar *category,
				       const gchar *manufacturer,
				       IrrecoStringTable **models)
{
	IrrecoStringTable *model_list;
	IrrecoStringTable *manufacturer_list;
	IRRECO_ENTER
	
	if (!irreco_webdb_cache_verify_category(self) ||
	    !irreco_webdb_cache_verify_manufacturer(self, category,
						    &manufacturer_list) ||
	    !irreco_webdb_cache_verify_model(self, manufacturer_list,
					     manufacturer, &model_list)) {
		IRRECO_RETURN_BOOL(FALSE)		
	}
		
	if(model_list == NULL) {
		gboolean success = FALSE;
		IrrecoWebdbClient *client = (IrrecoWebdbClient *) self->private;
		
		IRRECO_RETRY_LOOP_START(self->loop)
			if (irreco_webdb_cache_test(self) == FALSE) break;
			success = irreco_webdb_client_get_models(
				client, category, manufacturer, &model_list);
			
			if (success) break;
			irreco_webdb_client_get_error_msg(
				client, self->error_msg);
		IRRECO_RETRY_LOOP_END(self->loop)
		
		if (success == FALSE) IRRECO_RETURN_BOOL(FALSE);
		
		irreco_string_table_sort_abc (model_list);
		irreco_string_table_change_data(manufacturer_list, manufacturer,
						model_list);		
	}
	
	irreco_string_table_get(manufacturer_list, manufacturer,
				(gpointer *) models);
	IRRECO_RETURN_BOOL(TRUE)
}

/**
 * Fetches configs belonging given category, manufacturer and model.
 *
 * @return TRUE if configs are fetched succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_get_configs(IrrecoWebdbCache *self, 
				        const gchar *category,
	   			        const gchar *manufacturer,
	      			        const gchar *model,
					IrrecoStringTable **configs)
{
	IrrecoStringTable * config_list;
	IrrecoStringTable * model_list;
	IrrecoStringTable * manufacturer_list;
	IRRECO_ENTER

	if (!irreco_webdb_cache_verify_category(self) ||
	    !irreco_webdb_cache_verify_manufacturer(self, category,
						    &manufacturer_list) ||
	    !irreco_webdb_cache_verify_model(self, manufacturer_list,
					     manufacturer, &model_list) ||
	    !irreco_webdb_cache_verify_config(self, model_list, model,
					      &config_list)) {
		IRRECO_RETURN_BOOL(FALSE)		
	}
	
	if(config_list == NULL) {
		gboolean success = FALSE;
		IrrecoWebdbClient *client = (IrrecoWebdbClient *) self->private;
		
		IRRECO_RETRY_LOOP_START(self->loop)
			if (irreco_webdb_cache_test(self) == FALSE) break;
			success = irreco_webdb_client_get_configs(client,
				category, manufacturer, model, &config_list);
			
			if (success) break;
			irreco_webdb_client_get_error_msg(
				client, self->error_msg);
		IRRECO_RETRY_LOOP_END(self->loop)
		
		if (success == FALSE) IRRECO_RETURN_BOOL(FALSE);
		
		irreco_string_table_change_data(model_list, model, config_list);
	}
	
	irreco_string_table_get(model_list, model, (gpointer *) configs);
	IRRECO_RETURN_BOOL(TRUE)
}

/**
 * Fetches configuration by given ID.
 *
 * @return TRUE if configuration is fetched succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_get_configuration(IrrecoWebdbCache *self, 
					      gint id,
					      IrrecoWebdbConf **config)
{
	IrrecoWebdbConf	* configuration;
	IRRECO_ENTER
	
	if (g_hash_table_lookup(self->conf_hash, (gconstpointer) &id) == NULL) {
		gboolean success = FALSE;
		IrrecoWebdbClient *client = (IrrecoWebdbClient *) self->private;
		
		IRRECO_RETRY_LOOP_START(self->loop)
			if (irreco_webdb_cache_test(self) == FALSE) break;
			success = irreco_webdb_client_get_configuration(
				client, id, &configuration);
			
			if (success) break;
			irreco_webdb_client_get_error_msg(client,
							  self->error_msg);
		IRRECO_RETRY_LOOP_END(self->loop)

		if (success == FALSE) IRRECO_RETURN_BOOL(FALSE);
		
		g_hash_table_insert(self->conf_hash, 
				    (gpointer) &configuration->id,
				    (gpointer) configuration);
	}
	
	*config = g_hash_table_lookup(self->conf_hash, (gconstpointer) &id);
	
	IRRECO_RETURN_BOOL(TRUE)
}

/**
 * Fetches file by given hash and name.
 *.
 * @return TRUE if configuration is fetched succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_get_file(IrrecoWebdbCache *self, 
				     const gchar *file_hash,
	 			     const gchar *file_name,
				     GString **file_data)
{
	GString *file;
	gboolean success = FALSE;
	IrrecoWebdbClient *client;
	IRRECO_ENTER

	client = (IrrecoWebdbClient *) self->private;
	
	IRRECO_RETRY_LOOP_START(self->loop)
		if (irreco_webdb_cache_test(self) == FALSE) break;
		success = irreco_webdb_client_get_file(
			client, file_hash, file_name, &file);
		
		if (success) break;
		irreco_webdb_client_get_error_msg(client, self->error_msg);
	IRRECO_RETRY_LOOP_END(self->loop)

	if (success == FALSE) IRRECO_RETURN_BOOL(FALSE);
	
	*file_data = file;
	
	IRRECO_RETURN_BOOL(TRUE)
}

/**
 * Check if user with given name is already at DB.
 *
 *.@param user_exists will contain info wether user name was at DB.
 * @return TRUE if test is done succesfully, FALSE otherwise.
 */
gboolean irreco_webdb_cache_get_user_exists(IrrecoWebdbCache *self,
					const gchar *name,
					gboolean *user_exists)
{
	IrrecoWebdbClient *client;
	gboolean success = FALSE;
	
	IRRECO_ENTER
	client = (IrrecoWebdbClient *) self->private;	
	IRRECO_RETRY_LOOP_START(self->loop)
		/* test connection to webdb */
		if (irreco_webdb_cache_test(self) == FALSE) break;
	
		success = irreco_webdb_client_get_user_exists(client, 
							      name, 
							      user_exists);
		
		if (success) break;
		
		/* irreco_webdb_client_get_user_exists failed, get err msg */
		irreco_webdb_client_get_error_msg(client, self->error_msg);
	IRRECO_RETRY_LOOP_END(self->loop)
		
	if (success == FALSE) IRRECO_RETURN_BOOL(FALSE);
	
	IRRECO_RETURN_BOOL(TRUE)
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Events and Callbacks                                                       */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

